C-----------------------------------------------------------------------
C
C                        SYRTHES version 3.4
C                        -------------------
C
C     This file is part of the SYRTHES Kernel, element of the
C     thermal code SYRTHES.
C
C     Copyright (C) 1988-2008 EDF S.A., France
C
C     contact: syrthes-support@edf.fr
C
C
C     The SYRTHES Kernel is free software; you can redistribute it
C     and/or modify it under the terms of the GNU General Public License
C     as published by the Free Software Foundation; either version 2 of
C     the License, or (at your option) any later version.
C
C     The SYRTHES Kernel is distributed in the hope that it will be
C     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
C     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C
C     You should have received a copy of the GNU General Public License
C     along with the Code_Saturne Kernel; if not, write to the
C     Free Software Foundation, Inc.,
C     51 Franklin St, Fifth Floor,
C     Boston, MA  02110-1301  USA
C
C-----------------------------------------------------------------------
C                       *****************
                        SUBROUTINE CCOEQF
C                       *****************
C
     * ( VALEQ,NELRAY,NRFRAY,NFCFRA,PHFRAF,NGFPEF,IREF,NB )
C
C***********************************************************************
C* SYRTHES 3.4.2                                    COPYRIGHT EDF 2008 *
C***********************************************************************
C AUTEURS : C. PENIGUEL, I. RUPP                                       *
C***********************************************************************
C                                                                      *
C   FONCTION :                                                         *
C   --------   CODAGE DES CONDITIONS (LIMITES, PHYSIQUES,...)          *
C              DANS LES TABLEAUX ADAPTES                               *
C              Programme valable pour le conditions physiques          *
C              equivalente couple au fluide                            *
C                                                                      *
C                                                                      *
C-----------------------------------------------------------------------
C               (*)   (*)                 ARGUMENTS                    !
C   .________.______.____._____________________________________________.
C   !  NOM   ! TYPE !MODE!                  ROLE                       !
C   !________!______!____!_____________________________________________!
C   ! VAL    !   R  ! D  ! VALEUR DE LA CONDITION A APPLIQUER          !
C   ! NRFRAY !  TE  ! D  ! REFERENCES DES FACES DU MAILLAGE DE RAYONNE !
C   ! NFCFRA !   E  ! D  ! NOMBRE DE FACES DU MAILLAGE DE RAYONNEMENT  !
C   ! PHFRAF !  TR  ! R  ! PROPRIETES PHYSIQUES EQUIVQLENTE K,E,T,H    !
C   ! IREF   !  TE  ! D  ! REFERENCES SUR LESQUELLES IL FAUT APPLIQUER !
C   !        !      !    ! LA CONDITION "COND"                         !
C   ! NB     !   E  ! D  ! NOMBRE DE REFERENCES CONCERNEES             !
C   !________!______!____!_____________________________________________!
C   ! COMMONS                                                          !
C   !__________________________________________________________________!
C   !/NLOFES/!      ! D  !                                             !
C   !__________________________________________________________________!
C
C (*) TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
C     ET TYPES COMPOSES
C (*) MODE : D (DONNEE NON MODIFIEE), R (RESULTAT), M (DONNEE MODIFIEE)
C            A (TABLEAU AUXILIAIRE)
C-----------------------------------------------------------------------
C     SOUS PROGRAMME(S) APPELE(S) : --
C                                   
C-----------------------------------------------------------------------
C     SOUS PROGRAMME(S) APPELANT(S) : LECLIR
C
C***********************************************************************
C
      IMPLICIT NONE
C
C**********************************************************************
C     DONNEES EN COMMON 
C**********************************************************************
C
#include "nlofes.h"
#include "xrefer.h"
#include "optct.h"
#include "rayonn.h"
C
C**********************************************************************
C
C..Variables externes
      INTEGER NELRAY,NB,NFCFRA
      INTEGER NRFRAY(NELRAY),IREF(NRFMAX),NGFPEF(NFCFRA)
      DOUBLE PRECISION VALEQ(4),PHFRAF(NFCFRA,4)
C
C..Variables internes
      INTEGER N,N1,NR,NBRE,NUMREF
C
C**********************************************************************
C
C     0- Initialisations
C     ------------------
      NBRE = 0
C
C     1- Cas ou la condition s'applique sur tous les noeuds
C     -----------------------------------------------------
C
      IF (IREF(1) .EQ. -1 ) THEN
C
        DO 110 N=1,NFCFRA
           PHFRAF(N,1) = VALEQ(1)
           PHFRAF(N,2) = VALEQ(2)
           PHFRAF(N,3) = VALEQ(3)
           PHFRAF(N,4) = VALEQ(4)
           NBRE = NBRE + 1
  110  CONTINUE
C
      ELSE
C
C     2- Application de la condition aux references citees
C     ----------------------------------------------------
C
C
       DO 200 N1=1,NB
C
         NR = IREF(N1)
C
         DO 210 N=1,NFCFRA
C
           NUMREF = NRFRAY(NGFPEF(N))
C
           IF (NUMREF .EQ. NR) THEN
              PHFRAF(N,1) = VALEQ(1)
              PHFRAF(N,2) = VALEQ(2)
              PHFRAF(N,3) = VALEQ(3)
              PHFRAF(N,4) = VALEQ(4)
              NBRE = NBRE + 1
           ENDIF
C
  210   CONTINUE
  200  CONTINUE 
C
      ENDIF
C
C
C     3- Impressions de controle
C     --------------------------
C
      IF (NBLBLR.EQ.10) WRITE(NFECRA,3000) NBRE
C
C
C--------
C FORMATS
C--------
 3000 FORMAT('     CCOEQF : Rayonnement - Nombre de faces traitees',
     *        ' de type paroi equivalente et couplee au fluide:',I6)
C----
C FIN
C----
C
      RETURN
      END
